Attribute VB_Name = "FolderBrowse"

'        Allow the user to browse for a specific folder or printer.
'
Option Explicit

'-----------------------------------------------
' the various parameters you can use to control
'   what the user browses for.  for more info,
'   see the December 1997 Visual Basic
'   Programmer's Journal (www.windx.com).
'-----------------------------------------------
Private Type BROWSEINFO
    '-----------------------------------------------
    ' owner.
    Owner As Long
    '-----------------------------------------------
    ' root IDL; can be null.
    RootIdl As Long
    '-----------------------------------------------
    ' receives display name.
    DisplayName As String
    '-----------------------------------------------
    ' title to display.
    Title As String
    '-----------------------------------------------
    ' BIF constants, or 0.
    Flags As Long
    '-----------------------------------------------
    ' callback address.
    CallbackAddress As Long
    '-----------------------------------------------
    ' callback info.
    CallbackParam As Long
    '-----------------------------------------------
    ' index to the system image list
    Image As Long
End Type

'-----------------------------------------------
' only allow the user to browse for printers.
'-----------------------------------------------
Private Const BIF_BROWSEFORPRINTER = &H2000&

'-----------------------------------------------
' start the browsing at the desktop level.
'-----------------------------------------------
Private Const CSIDL_DESKTOP = &H0

'-----------------------------------------------
' API calls.
'-----------------------------------------------
Private Declare Function SHGetSpecialFolderLocation Lib "Shell32.DLL" (ByVal hwndOwner As Long, ByVal Folder As Long, ByRef idl As Long) As Long
Private Declare Function SHGetPathFromIDList Lib "Shell32.DLL" (ByVal idl As Long, ByVal Path As String) As Long
Private Declare Function SHBrowseForFolder Lib "Shell32.DLL" (ByRef bi As BROWSEINFO) As Long
Private Declare Function GlobalFree Lib "kernel32" (ByVal hMem As Long) As Long
'
' Sub BrowseFolders
'   (
'   )
'
' returns:  the folder the user chose, or "" if the user cancelled.
'
' this function lets the user browse for a file.
'
' R0  28 Feb 98  MichaelHu    Humbug_Reality@csi.com
' R1  13 Oct 99  MichaelHu    Added argument strDlgTtl in this version.
'
Public Function BrowseFolders(Optional strDlgTtl As String = "Select folder") As String
    BrowseFolders = DoBrowse(0, strDlgTtl)
End Function
'
' Sub BrowsePrinters
'   (
'   )
'
' returns:  the printer the user chose, or "" if the user cancelled.
'
' this function lets the user browse for a printer.
'
' R0  28 Feb 98  MichaelHu
'
Public Function BrowsePrinters() As String
    BrowsePrinters = DoBrowse(BIF_BROWSEFORPRINTER, "Browse for Printers")
End Function
'
' Sub DoBrowse
'   (
'   flags specifying what can be browsed.
'   inFlags     As  Long
'   the title to use for the Browse dialog.
'   inTitle     As  String
'   )
'
' returns:  the object the user chose, or "" if the user cancelled.
'
' this function lets the user browse for a folder, printer, or other
'   system resource.  the 'inFlags' parameter determines what the user
'   can browse for; see the Windows API help for SHBrowseForFolder for
'   more info.
'
' R0  28 Feb 98  MichaelHu
'
Private Function DoBrowse(ByVal inFlags As Long, ByVal inTitle As String) As String

    On Error GoTo ERR_HANDLER
    '-----------------------------------------------
    ' assume failure.
    DoBrowse = ""
    
    '-----------------------------------------------
    ' the maximum string length.
    Const MAX_LEN = 260
    
    '-----------------------------------------------
    ' if we can get a reference to the Desktop
    '   object,
    Dim theIdl As Long
    If (SHGetSpecialFolderLocation(0, CSIDL_DESKTOP, theIdl) = 0) Then
        '-----------------------------------------------
        ' set up the BrowseInfo information
        Dim bif As BROWSEINFO
        With bif
            .Owner = 0
            .RootIdl = theIdl
            .DisplayName = Space(MAX_LEN)
            .Title = inTitle
            .Flags = inFlags
        End With
        '-----------------------------------------------
        ' and let the user browse.
        theIdl = SHBrowseForFolder(bif)
        '-----------------------------------------------
        ' if the user chose something,
        Dim folderPath As String
        If (theIdl <> 0) Then
            '-----------------------------------------------
            ' attempt to turn the ID into a full path;
            '   otherwise, use the filename from the
            '   BrowseInfo data.
            folderPath = Space(MAX_LEN)
            If (CBool(SHGetPathFromIDList(theIdl, folderPath))) Then
                DoBrowse = TrimStringAtNull(folderPath)
            Else
                DoBrowse = TrimStringAtNull(folderPath = bif.Title)
            End If
        End If
        '-----------------------------------------------
        ' BrowseForFolder() allocates some memory, but
        '   it's up to us to free it.
        Call GlobalFree(theIdl)
    End If
    
    Exit Function

ERR_HANDLER:
    Err.Raise Err.Number, "FolderBrowse:DoBrowse", Err.Description
    Resume ERR_END
    
ERR_END:
End Function
'
' Sub TrimStringAtNull
'   (
'   the string to trim.
'   value   As  String
'   )
'
' returns:  the input string truncated at the first null character.
'
' this function truncates a string at its first null character.
'
' R0  28 Feb 98  MichaelHu
'
Private Function TrimStringAtNull(ByVal value As String) As String

    On Error GoTo ERR_HANDLER
    
    '-----------------------------------------------
    ' the location of the first null character.
    Dim loc As Integer
    
    '-----------------------------------------------
    ' look for a null character.
    loc = InStr(value, vbNullChar)
    Select Case loc
        '-----------------------------------------------
        ' if it is anywhere after the first character,
        Case Is > 1
            '-----------------------------------------------
            ' trim the string at its location.
            TrimStringAtNull = VBA.Left$(value, loc - 1)
        '-----------------------------------------------
        ' it is isn't found,
        Case 0
            '-----------------------------------------------
            ' there's nothing to do.
            TrimStringAtNull = loc
        '-----------------------------------------------
        ' if it's the first character,
        Case 1
            '-----------------------------------------------
            ' we have an empty string.
            TrimStringAtNull = ""
    End Select
    
    Exit Function

ERR_HANDLER:
    Err.Raise Err.Number, "FolderBrowse:TrimStringAtNull", Err.Description
    Resume ERR_END
    
ERR_END:
End Function


